/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Yahoo! Mail Notifier code.
 *
 * The Initial Developer of the Original Code is
 * Michael Kraft.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var Cc = Components.classes;
var Ci = Components.interfaces;

var ym_login = {
	newUserName : null,
	newPassword : null,
	currentDefaultUser : null,
	currentDefaultUserPw : null,
	attemptedLogin : false,
	checking : false,
	ymNotifier : null,

	//
	// Retrieve the user preferences
	//
	doLoad : function() {
		// See if user is currently checking for mail
		var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
		var browsers=wm.getEnumerator('navigator:browser');
		while(browsers.hasMoreElements()) { 
			var win = browsers.getNext();
			if (win.gYm_notifier) { 
				this.ymNotifier = win.gYm_notifier;
			    if (win.gYm_notifier.timer) {
					this.checking = true;
					break;
				}
			}
		}
		
		var menulist = document.getElementById("ym-notifier-username");
		var userName = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
		var users = ym_prefs.getPref(ym_prefs.PREF_USERS).split(" ");
		for (var i=0; i < users.length; i++) {
			menulist.appendItem(users[i]);
		}

		if (userName) {
			this.currentDefaultUser = userName;
			document.getElementById("ym-notifier-username").value = userName;
			ym_prefs.loadUser(userName);
			if (ym_prefs.user.userName) {
				this.currentDefaultUserPw = ym_prefs.user.password;
				document.getElementById("ym-notifier-password").value = ym_prefs.user.password;
				document.getElementById("ym-notifier-remember-pw").checked = ym_prefs.user.remember_pw;
				document.getElementById("ym-notifier-override").checked = ym_prefs.user.overrideURL;
				document.getElementById("ym-notifier-mailURL").value = ym_prefs.user.mailURL;
			}
		}
		// Can't disable default, must create a new default and enable it there
		document.getElementById("ym-notifier-make-default").checked = true;
		document.getElementById("ym-notifier-make-default").setAttribute("disabled",true);

		if (!document.getElementById("ym-notifier-username").value || 
		    !document.getElementById("ym-notifier-password").value) {
			document.getElementById("login").setAttribute("disabled",true);
			document.getElementById("ym-delete").setAttribute("disabled",true);
		}

		// Enable/disable mailurl field
		this.toggleMailURL();

		return true;
	},

	doOK : function() {
		this.newUserName = document.getElementById("ym-notifier-username").value;
		this.newPassword = document.getElementById("ym-notifier-password").value;

		// username and or password changed
		if ((this.newUserName != this.currentDefaultUser) ||
		    (this.newPassword != this.currentDefaultUserPw)) {

			this.attemptedLogin = true;
		
			// Call login to check login info, if login is successful we will get a login
			// notification, otherwise we will get a logout notification
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
													
			// logout first before we add observer because the login function might logout first
			this.ymNotifier.logout(this.currentDefaultUser);
															
			observerService.addObserver(this,"ym-gui-change",false);	 

			this.ymNotifier.login(this.newUserName, this.newPassword, 
			                      document.getElementById("ym-notifier-remember-pw").checked);
			return false;
		}
		// no username/password change so if not checking login
		else if (!this.checking) {
			this.ymNotifier.login(this.newUserName, this.newPassword, 
			                      document.getElementById("ym-notifier-remember-pw").checked);
		}
		// otherwise just save updated data.
		else this.saveUser(this.newUserName);
		return true;
	},

	//
	// Observer functions
	//  
	observe : function(aSubject, aTopic, aData) {
	
		if (aTopic == "ym-gui-change") {
			aData = aData.split(" ");
		
			if (aData[0] == "login") {
				// give time for other observers to handle login as well
				setTimeout(function() {ym_login.saveUser(aData[1]);}, 0);
			}
			else if (aData[0] == "logout") {
				this.doCleanup();
			}
		}
	},

	//
	// Canceled
	//
	doCancel : function() {
		this.doCleanup();
	
		// if user tried to change users and failed log back in with default user
		if (this.attemptedLogin && this.checking) this.ymNotifier.doLogin();
		return true;
	},

	//
 	// If user logged in successfully store info
	//
	saveUser : function(username) {
	
		// if this username is the one that logged in
		if (this.newUserName == username) {
			ym_prefs.saveUser(this.newUserName, this.newPassword, 
			                  document.getElementById("ym-notifier-remember-pw").checked,
			                  document.getElementById("ym-notifier-override").checked,
			                  document.getElementById("ym-notifier-mailURL").value,
			                  document.getElementById("ym-notifier-make-default").checked);

			this.doCleanup();
			if (!document.getElementById("ym-notifier-make-default").checked) {
				this.ymNotifier.logout(this.newUserName);
				
				// log default user back in if he was checking
				if (this.checking) this.ymNotifier.doLogin();
			}
			else if (this.newUserName != this.currentDefaultUser) {
				// notify all windows of login because they had the wrong default user at the time
				var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
				observerService.notifyObservers(null,"ym-gui-change","login " + this.newUserName);
			}				
			setTimeout("window.close()", 0)
		}
	},

	//
	// Cleanup any observers
	//
	doCleanup : function() {
		// Remove observers
		try {
			var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
			observerService.removeObserver(this,"ym-gui-change"); 	
		}
		catch (ex) {}
	},

	//
	// Delete the user info
	//
	deleteUser : function() {
	
		if (confirm(document.getElementById("ym-notifierStrings").getString("confirmdelete"))) {
 
			var menulist = document.getElementById("ym-notifier-username");

			// if deleting default user logout
			if (menulist.value == this.currentDefaultUser) {
				this.ymNotifier.logout(this.currentDefaultUser);
				this.checking = false;
			}

			ym_prefs.deleteUser(menulist.value);
			menulist.removeItemAt(menulist.selectedIndex);
			menulist.selectedIndex = 0;

			this.userNameChanged();
		}
	},

	//
	// Retreive password and remember password settings on username change
	//
	userNameChanged : function() {
		var userName = document.getElementById("ym-notifier-username").value;
		var foundInfo = false;

		try {
			if (userName) {
				ym_prefs.loadUser(userName);
				if (ym_prefs.user.userName) {
					var currentDefaultUser = ym_prefs.getPref(ym_prefs.PREF_DEFAULT_USER);
					document.getElementById("ym-notifier-password").value = ym_prefs.user.password;
					document.getElementById("ym-notifier-remember-pw").checked = ym_prefs.user.remember_pw;
					document.getElementById("ym-notifier-override").checked = ym_prefs.user.overrideURL;
					document.getElementById("ym-notifier-mailURL").value = ym_prefs.user.mailURL;
					document.getElementById("ym-notifier-make-default").checked = (userName == currentDefaultUser);
					document.getElementById("ym-notifier-make-default").setAttribute("disabled",(userName == currentDefaultUser));
					document.getElementById("ym-delete").setAttribute("disabled",false);
					foundInfo = true;
				}
			}
		}
		catch (ex) { dump(ex + "\n"); }
		finally {
			if (!foundInfo) {
				document.getElementById("ym-notifier-password").value = "";
				document.getElementById("ym-notifier-remember-pw").checked = false;
				document.getElementById("ym-notifier-override").checked = false;
				document.getElementById("ym-notifier-mailURL").value = CONST_DEFAULT_MAILURL;
				document.getElementById("ym-notifier-make-default").checked = !this.currentDefaultUser;
				document.getElementById("ym-notifier-make-default").setAttribute("disabled",!this.currentDefaultUser);
				document.getElementById("ym-delete").setAttribute("disabled",true);
			}

			this.updateLoginButton();
		}
	},

	//
	// Enable/Disable login button 
	//
	updateLoginButton : function() {
		if (document.getElementById("ym-notifier-username").value && 
		    document.getElementById("ym-notifier-password").value)
			document.getElementById("login").setAttribute("disabled",false);
		else 
			document.getElementById("login").setAttribute("disabled",true);
	},

	//
	// Disable/Enable override mail URL
	//
	toggleMailURL : function() {
		if (document.getElementById("ym-notifier-override").checked)
			document.getElementById("ym-notifier-mailURL").removeAttribute("disabled");
		else
			document.getElementById("ym-notifier-mailURL").setAttribute("disabled", true);
	},
}

//var gYm_login = new ym_login();
